/* VGAlib version 1.2 - (c) 1993 Tommy Frandsen 		   */
/*								   */
/* This library is free software; you can redistribute it and/or   */
/* modify it without any restrictions. This library is distributed */
/* in the hope that it will be useful, but without any warranty.   */

#ifndef VGA_H
#define VGA_H

#ifdef __cplusplus
extern "C" {
#endif


#define TEXT 	     0		/* VGAlib modes */
#define G320x200x16  1
#define G640x200x16  2
#define G640x350x16  3
#define G640x480x16  4
#define G320x200x256 5
#define G320x240x256 6
#define G320x400x256 7
#define G360x480x256 8
#define G640x480x2   9

#define G640x480x256 10		/* Compatible with VGAlib v1.2 */
#define G800x600x256 11
#define G1024x768x256 12

#define G320x200x32K 13
#define G320x200x64K 14
#define G320x200x16M 15
#define G640x480x32K 16
#define G640x480x64K 17
#define G640x480x16M 18
#define G800x600x32K 19
#define G800x600x64K 20
#define G800x600x16M 21
#define G1024x768x32K 22
#define G1024x768x64K 23
#define G1024x768x16M 24
#define G1280x1024x256 25
#define G1280x1024x32K 26
#define G1280x1024x64K 27
#define G1280x1024x16M 28

#define G720x350x16  29		/* standard VGA modes from Hartmut Schirmer */
#define G720x480x16  30

#define GLASTMODE 30

extern int vga_setmode(int mode);
extern int vga_hasmode(int mode);
extern int vga_setflipchar(int c);

extern int vga_clear();
extern int vga_flip();

extern int vga_getxdim();
extern int vga_getydim();
extern int vga_getcolors();

extern int vga_setpalette(int index, int red, int green, int blue);
extern int vga_getpalette(int index, int *red, int *green, int *blue);
extern int vga_setpalvec(int start, int num, int *pal);
extern int vga_getpalvec(int start, int num, int *pal);

extern int vga_screenoff();
extern int vga_screenon();

extern int vga_setcolor(int color);
extern int vga_drawpixel(int x, int y);
extern int vga_drawline(int x1, int y1, int x2, int y2);
extern int vga_drawscanline(int line, unsigned char* colors);
extern int vga_drawscansegment(unsigned char* colors, int x, int y, int length);

extern int vga_getch();

extern int vga_dumpregs();


/* Extensions to VGAlib v1.2: */

extern int vga_setrgbcolor( int r, int g, int b );

#define HAVE_BITBLIT 1
#define HAVE_FILLBLIT 2
#define HAVE_IMAGEBLIT 4

typedef struct {
	int width;
	int height;
	int bytesperpixel;
	int colors;
	int linewidth;		/* scanline width in bytes */
	int maxlogicalwidth;	/* maximum logical scanline width */
	int startaddressrange;	/* changeable bits set */
	int maxpixels;		/* video memory / bytesperpixel */
	int haveblit;		/* mask of blit functions available */
} vga_modeinfo;

extern vga_modeinfo *vga_getmodeinfo( int mode );
extern int vga_getdefaultmode();
extern int vga_getcurrentmode();
extern int vga_getcurrentchipset();
extern char *vga_getmodename( int mode );
extern int vga_getmodenumber( char *name );

extern unsigned char *graph_mem;
extern void vga_setpage( int p );
extern void vga_setlogicalwidth( int w );
extern void vga_setdisplaystart( int a );
extern void vga_waitretrace();
extern int vga_claimvideomemory( int k );
extern void vga_disabledriverreport();

#define UNDEFINED	0
#define VGA		1
#define ET4000		2
#define CIRRUS		3
#define TVGA8900	4 
#define OAK		5

extern void vga_setchipset( int c );
extern void vga_setchipsetandfeatures( int c, int par1, int par2 );
extern void vga_gettextfont( void *font );
extern void vga_puttextfont( void *font );
extern void vga_settextmoderegs( void *regs );
extern void vga_gettextmoderegs( void *regs );

extern void vga_bitblt( int srcaddr, int destaddr, int w, int h, int pitch );
extern void vga_imageblt( void *srcaddr, int destaddr, int w, int h, int pitch );
extern void vga_fillblt( int destaddr, int w, int h, int pitch, int c );
extern void vga_hlinelistblt( int ymin, int n, int *xmin, int *xmax, int pitch, int c );

#ifdef __cplusplus
}
#endif

#endif /* VGA_H */

